#include "RectangleNode.h"

#include "Rectangle.h"

#include "MeshPrimitives.h"

#include <iostream>

RectangleNode::RectangleNode(const DiscretePoint& loc, unsigned width, unsigned height)
    : GeometryNode(loc),
      mWidth(width),
      mHeight(height)
{

}

std::unique_ptr<RectangleNode> RectangleNode::Create(const DiscretePoint& loc, unsigned width, unsigned height)
{
    return std::make_unique<RectangleNode>(loc, width, height);
}

GeometryNode::Type RectangleNode::getType()
{
    return GeometryNode::Type::Rectangle;
}

unsigned RectangleNode::getWidth() const
{
    return mWidth;
}

unsigned RectangleNode::getHeight() const
{
    return mHeight;
}

void RectangleNode::setWidth(unsigned width)
{
    if (mWidth != width)
    {
        mTransformIsDirty = true;
        mWidth = width;
    }
}

void RectangleNode::setHeight(unsigned height)
{
    if (mHeight != height)
    {
        mTransformIsDirty = true;
        mHeight = height;
    }
}

void RectangleNode::setLocation(const DiscretePoint& loc)
{
    if (mLocation != loc)
    {
        mTransformIsDirty = true;
        mLocation = loc;
    }
}

void RectangleNode::update(FontsManager* fontsManager)
{
    if (!mSceneItem || mGeometryIsDirty)
    {
        const auto rect = Rectangle(Point(), 1, 1);
        auto mesh = MeshPrimitives::build(rect);

        if (!mSceneItem)
        {
            mSceneItem = std::make_unique<SceneModel>(std::move(mesh));
            mSceneItem->setName(mName + "_Model");
        }
        else
        {
            dynamic_cast<SceneModel*>(mSceneItem.get())->updateMesh(std::move(mesh));
        }
        mGeometryIsDirty = false;
    }

    if (mTransformIsDirty)
    {
        mSceneItem->updateTransform({mLocation, static_cast<double>(mWidth), static_cast<double>(mHeight)});
        mTransformIsDirty = false;
    }

    if (mMaterialIsDirty)
    {
        mSceneItem->updateUniformColor(mFillColor);
        mMaterialIsDirty = false;
    }
}
